# Requires
require 'open-uri'
require 'uri'
require 'Core/Base'
require 'Core/Exceptions'
require 'Utils/FastestURL'

module PackerFiles

   module Core

      # Base class for CDImage implmentation
      class CDImageImpl

	 # Return back the ISO name (It can be a regex pattern)
	 def get_iso_name_pattern
	    '.*'
	 end

	 # Return back the Checksum File Pattern (It can be a regex pattern)
	 def get_check_sum_pattern
	    'MD5SUM'
	 end
    
	 # Return back the Checksum File Pattern (It can be a regex pattern)
	 def get_check_sum_type
	    'md5'
	 end

	 # Return the list of index URLs.
	 def get_index_urls
	    []
	 end
      end

      # Used for figuring out the URLs and Checksum values for the installer images
      # of debian.
      class CDImage < Base
      
	# Accessors that can be set.
	attr_accessor :arch
	attr_accessor :release
	attr_accessor :iso_name_pattern
	attr_accessor :check_sum_pattern
	attr_accessor :check_sum_type
       
	# Accessors that can be derived if not set 
	attr_accessor :index_urls
	attr_accessor :iso_url
	attr_accessor :check_sum

	# Implementation class for CD Image.
	attr_writer   :impl

	# Documentation for this class
	def self.doc_file
	   PackerFiles.DirPath('Core/example/CDImage.txt').first
	end

	# Constructor that yields self for accessor settings.
	def initialize
	   yield self if block_given?
	end

	# Overriden normalize
	def normalize
	  
	   # Do some basic checks.
	   raise NilException.new(self, 'arch')    if @arch.nil?
	   raise NilException.new(self, 'release') if @release.nil?

	   # Get everything else.
	   set_iso_url_and_checksum
	end

	# Compute the ISO URL and Check SUM for the CD Image, but only if it is
	# not defined already.
	private
	def set_iso_url_and_checksum
	   if @iso_url.nil? || @check_sum.nil?
	      set_iso_name_checksum_pattern
	      index_url, valid   = fastest_url
	      if (valid)
		(@iso_url, ck_url) = get_iso_checksum_url(index_url)
		@check_sum         = get_check_sum(ck_url)
	      else
		@iso_url        = URI::join(index_url, @iso_name_pattern).to_s
		@check_sum_type = 'none'
		@check_sum      = ''
	      end
	   end
	   raise NilException.new(self, 'check_sum_type')    if @check_sum_type.nil?
	end

	# Compute the ISO name of the CD Image from other variables, but only
	# if it is not defined already.
	def set_iso_name_checksum_pattern
	   if @iso_name_pattern.nil? && !@impl.nil?
	      name              = @impl.get_iso_name_pattern 
	      @iso_name_pattern = name.gsub('@release@', @release).gsub('@arch@', @arch)
	   end
	   if @check_sum_pattern.nil? && !@impl.nil?
	      name      = @impl.get_check_sum_pattern
	      @check_sum_pattern = name.gsub('@release@', @release).gsub('@arch@', @arch)
	   end
	   if @check_sum_type.nil? && !@impl.nil?
	      @check_sum_type    = @impl.get_check_sum_type
	   end
	   
	   # Raise more errors if implementation does not fill values.
	   raise NilException.new(self, 'iso_name_pattern')  if @iso_name_pattern.nil?
	   raise NilException.new(self, 'check_sum_pattern') if @check_sum_pattern.nil?
	   raise NilException.new(self, 'check_sum_type')    if @check_sum_type.nil?
	end

	# Return the MD5 Checksum for the ISO URL
	def get_check_sum(ck_url)
	  name = URI::parse(@iso_url).path.split('/').last
	  begin
	     open(ck_url) {|f|
		f.each_line do |line|
		   if match = line.match(name)
		      return line.split(' ').first
		   end
		end
	     }
	  rescue OpenURI::HTTPError => e
	     raise URLException.new(ck_url, e)
	  end
	end

	# Return the full URL for the CD Image.
	private
	def get_iso_checksum_url(index_url)
	  iso_regex = "href=\"(#{@iso_name_pattern})\""
	  ck_regex  = "href=\"(#{@check_sum_pattern})\""
	  iso_url   = ''
	  ck_url    = ''	
	  open(index_url) {|f|
	     f.each_line do |line|
	       if match = line.match(iso_regex)
		  iso_url = URI::join(index_url, match[1]).to_s
	       end
	       if match = line.match(ck_regex)
		  ck_url = URI::join(index_url, match[1]).to_s
	       end
	     end
	  }
	  raise "Cannot find file matching #{@iso_name_pattern}"  if iso_url.empty?
	  raise "Cannot find file matching #{@check_sum_pattern}" if ck_url.empty?
	  return iso_url, ck_url
	end

	# Get the Fastest Index URL from the list of curated index URLs
	def fastest_url

	   # Slow URL = 5 seconds latency with a trial = 1
	   f    = Utils::FastestURL.new(5.0, 1)
	   urls = curated_index_urls
	   raise EmptyArrayException.new(self, 'index_urls') if urls.empty?
	   all = f.best_urls(urls)
	   if all.empty?
	      return urls.first, false
	   else
	      return all.flatten.first, true
	   end
	end

	# Curate the URL list by replacing '@release@' with @release and '@arch'
	# with @arch.
	def curated_index_urls
	   @index_urls ||= @impl.index_urls if !@impl.nil?
	   raise NilException.new(self, 'index_urls')        if @index_urls.nil?
	   raise EmptyArrayException.new(self, 'index_urls') if @index_urls.empty?
	   @index_urls.map! {|url|
	      url.gsub('@release@', @release).gsub('@arch@', @arch)
	   }
	end

      end 

      
   end # Core

end # PackerFiles
