# Requires
require 'Core/CDImage'
require 'Core/Exceptions'
require 'Virtual/Hypervisors'
require 'Provision/Provisioners'
require 'Utils/Generator'
require 'Utils/TypeAccessor'

module PackerFiles

  # The Builder class is used as the base for building OS classes 
  # that generate all the files required for Packer to work properly for
  # a given OS
  class Builder < Utils::TypeAccessor

    # attr_accessor for output directory
    attr_accessor :OutDir

    # Type accessors for mandatory common objects
    type_accessor Core::CDImage
    type_accessor Virtual::Hypervisors

    # Type accessor for optional Provisioners.
    type_accessor Provisioners, 'Provision', true

    # Versions supported by this class
    def self.versions
       []
    end

    # Name of this class as exposed to others
    def name
       self.class.name.gsub('PackerFiles::','')
    end

    # Name of the http directory
    def http_dir
      File.join(@OutDir, 'http')
    end

    # Name of the top level JSON file
    def packer_json_file
      rel  = self.CDImage.release
      arch = self.CDImage.arch
      file = self.name.gsub('::', '-') + "-#{rel}-#{arch}.json"
      File.join(@OutDir, file)
    end

    # Base class function that needs to be called first to handle
    # parameter validation.
    def Generate
      if @OutDir.nil?
         raise ArgumentError.new('Need to provide output directory')
      end

      # Track through all the accessors in the ancestry tree and check
      # if non-optional type accessors are missing.
      index     = self.class.ancestors.index(Builder)
      ancestors = self.class.ancestors.slice(0, index + 1 )
      ancestors.each do |klass|
	 hash  = klass.type_accessors
	 next  if hash.nil?
         hash.each_pair do |func, type_data|
            opt = type_data.last
            obj = self.method(func).call
            raise Utils::MissingAccessor.new(func, self.name) if obj.nil? && !opt
         end
      end

      # Normalize CD Image by calling the cd image hook of the derived class.
      cd_image_hook
      self.CDImage.normalize

      # Call Hook for derived classes to add their functionality.
      generate_hook
	
      # Generate Packer File
      generate_packer_file

    end

    # Generate the Various Packer File components
    private
    def generate_packer_file

      # Manage various packer file components via the generator
      gen = Utils::Generator.new

      # handle hypervisors
      self.Hypervisors.convert(gen, self)

      # handle provisioners
      self.Provision.convert(gen, self) if !self.Provision.nil?

      # Finally generate the JSON for packer
      gen.create_json(packer_json_file)

    end

    # Various hook functions
    def generate_hook; end
    def cd_image_hook; end
    def vbox_converter_hook(gen); end
    def vmware_converter_hook(gen); end
    def kvm_converter_hook(gen); end 
    def boot_command; end
    def shutdown_command; end
    def guest_os_type; end
    def vmware_guest_os_type; end
    def vmware_guest_os_flavor; end
    def run_command_as_root;   end
    def enable_root_for_user; end
    def disable_root_for_user; end

  end
end
