# Requires
require 'OS/Builder'
require 'Core/Locale'
require 'Core/Keyboard'
require 'Core/TimeZone'
require 'Core/User'
require 'Core/Disk'
require 'Core/Network'
require 'Core/Packages'
require 'Core/CDImage'
require 'Virtual/VirtualBoxConverter'
require 'Virtual/KVMConverter'
require 'Virtual/VMWareConverter'
require_relative 'Kickstart'
require_relative 'CD'

# OS class that handles details for generating Packer files for CentOS
# Servers.
module PackerFiles
 module CentOS

   class Server < Builder

     # List of type accessors used by this class that also gets converted
     # to a preseed
     def self.preseed_accessors
	[Core::Locale, Core::Keyboard, Core::TimeZone, 
         Core::User,   Core::Disk,     Core::Network, 
         Core::Packages
        ]
     end

     # Create accessors based on Type
     self.preseed_accessors.each do |type|
     	type_accessor type
     end

     # Constructor. Yield self for further initialization
     def initialize
	yield self if block_given?
     end 

     # hook function for generation
     def generate_hook

	# Construct objects that needs to be converted by using accessors.
        # The tracker type_accessors tracks the mapping b/w Type names and
        # method calls.
	objects =  []
	objects.push self.Locale, self.Keyboard, self.TimeZone, self.User
	objects.push self.Disk,   self.Network,  self.Packages
	objects.each do |obj|
	   obj.normalize
        end

	# Call a hook function to customize kickstart generation by
	# derived classes.
	kickstart_write_hook(objects)
     end

     # Hook function called before normalizing CD Image
     def cd_image_hook
	self.CDImage.impl = CD.new
     end 

     # Hook function called for converting this object into a hash for
     # VirtualBox
     def vbox_converter_hook(gen)
        Virtual::VirtualBoxConverter.new(self, gen).value
     end

     # Hook function called for converting this object into a hash for
     # KVM
     def kvm_converter_hook(gen)
        Virtual::KVMConverter.new(self, gen).value
     end

     # Hook function called for converting this object into a hash for
     # VMWare
     def vmware_converter_hook(gen)
        Virtual::VMWareConverter.new(self, gen).value
     end

     # Hook for writing into preseed file.
     def kickstart_write_hook(objects)

	# Preseed file by evaluating templates @ Debian folder.
	kickstart = Kickstart.new(kickstart_file)

	# Evaluate Head 
	kickstart.eval_file('CentOS/Templates/Base_erb.rb', {})

        # Convert objects into Preseed configuration.
        objects.each do |obj|
	   kickstart.convert obj, 'CentOS/Templates'
        end

	# Evaluate tail
	kickstart.eval_file('CentOS/Templates/Tail_erb.rb', {'obj' => self})

     end

     # Name of the preseed file
     def kickstart_file
        rel  = self.CDImage.release
        arch = self.CDImage.arch
        name = "kickstart-centos-server-#{rel}-#{arch}.cfg"
	File.join(http_dir, name)
     end

     # Boot Command Override
     def boot_command
        file  = PackerFiles.DirPath('CentOS/Templates/boot_command_erb.rb')
	hash  = { 'obj' => self, 'network' => self.Network }
        value = PackerFiles.evaluate_erb(file.first, hash)
	value.split("\n")
     end

     # Shutdown command Override
     def shutdown_command
        file  = PackerFiles.DirPath('CentOS/Templates/shutdown_command_erb.rb')
        hash  = { 'obj' => self.User }
        value = PackerFiles.evaluate_erb(file.first, hash)
	value.chop
     end

     # Guest OS Type override
     def guest_os_type
	return "RedHat_64" if (self.CDImage.arch == "x86_64")
	return "RedHat"
     end

     # VMWare Guest OS Type override
     def vmware_guest_os_type
	return "rhel4-64" if (self.CDImage.arch == "x86_64")
	return "rhel4"
     end

     # VMWare Guest Flavor override
     def vmware_guest_os_flavor
	return "linux"
     end

     # Run any given command as root
     def run_command_as_root
        file  = PackerFiles.DirPath('CentOS/Templates/command_as_root_erb.rb')
        hash  = { 'obj' => self }
        value = PackerFiles.evaluate_erb(file.first, hash)
	value.chop
     end 

     # Enable root user command w/o a password
     def enable_root_for_user
        file  = PackerFiles.DirPath('CentOS/Templates/enable_root_erb.rb')
        hash  = { 'obj' => self }
        value = PackerFiles.evaluate_erb(file.first, hash)
	value.chop
     end

     # Disable root user command w/o a password
     def disable_root_for_user
        file  = PackerFiles.DirPath('CentOS/Templates/disable_root_erb.rb')
        hash  = { 'obj' => self }
        value = PackerFiles.evaluate_erb(file.first, hash)
	value.chop
     end

  end
 end
end 
