# Requires
require 'open-uri'
require 'benchmark'
require 'uri'

module PackerFiles
  
   module Utils

      # Given a list of URL, returns the fastest URL (to access) among them. 
      # This is done by launching several threads in parallel and estimating
      # the time taken to access the contents of these URLs.
      class FastestURL

	 # Error URL 
	 @@error = 25000
     
	 # Creates a new object of type FastestURL. slow_limit is used for 
	 # signalling the response time (in seconds) of a web URL to be 
	 # categorized as slow. Trials specifies the # of attempts the web
	 # URLs must be opened and closed.  
	 def initialize(slow_limit = 0.5 , trials = 1)
	    @limit  = slow_limit
	    @trials = trials
	 end 

	 # Returns URLs sorted in terms of best access times from a list of 
	 # input URLs.
	 def best_urls(url_list)
	    threads = url_list.map  { |h| Thread.new {speed_trials(h) }}
	    speeds  = threads.map   { |t| t.join; t.value }
	    hash    = Hash.new
	    url_list.each_with_index do |url, index|
	       hash[url] = speeds.at(index) if speeds.at(index) != @@error
	    end
	    hash.sort_by {|url, speed| speed} 
	 end

	 # Compute the latency of a web url, multiple times via independent
	 # trials. This is called in thread context only.
	 private
	 def speed_trials(url)
	   total = 0.0
	   @trials.times do
	     speed = estimate_speed(url)
	     if speed >= @limit
	       return @@error
	     else
	       total += speed
	     end
	   end
	   return (total / @trials)
	 end
	 
	 # This function is called in thread context only. It computes
	 # the  time it takes for the various mirrors to return the index
	 # as a proxy for measuring it's speed.
	 private
	 def estimate_speed(url)
	    Benchmark.realtime do
	      begin
		open(url) {|f| f.read}
	      rescue Exception => e
		return @@error
	      end
	    end
	 end

      end # FastestURL

   end # Utils
 
end # PackerFiles
