# Requires
require 'Core/Hypervisor'
require 'Utils/HashSerializer'
require 'Utils/Size'

module PackerFiles

   module Virtual

      # Abstraction for KVM(QEMU) builder in Packer. Refer to Packer.io
      # documentation on more details on the various variables for 
      # QEMU ISO Builder.
      class KVM < Utils::HashSerializer

	 # Import Size with KVM class as a Mix-in
	 include Utils::Size

	 # List of attributes that would be converted into Hash Values. The Hash
	 # values will eventually be converted into JSON values.
	 hash_variable :iso_checksum
	 hash_variable :iso_checksum_type
	 hash_variable :iso_url
	 hash_variable :ssh_username
	 hash_variable :accelerator
	 hash_variable :boot_command, Array
	 hash_variable :boot_wait
	 hash_variable :disk_size,    Integer
	 hash_variable :disk_interface
	 hash_variable :floppy_files, Array
	 hash_variable :format
	 hash_variable :headless
	 hash_variable :http_directory
	 hash_variable :http_port_min, Integer
	 hash_variable :http_port_max, Integer
	 hash_variable :iso_urls, Array
	 hash_variable :net_device
	 hash_variable :output_directory
	 hash_variable :qemuargs, Array
	 hash_variable :qemu_binary
	 hash_variable :shutdown_command
	 hash_variable :shutdown_timeout
	 hash_variable :ssh_host_port_min, Integer
	 hash_variable :ssh_host_port_max, Integer
	 hash_variable :ssh_key_path
	 hash_variable :ssh_password
	 hash_variable :ssh_port, Integer
	 hash_variable :ssh_wait_timeout
	 hash_variable :vm_name
	 hash_variable :type
	 hash_variable :vnc_port_min
	 hash_variable :vnc_port_max
	 hash_variable :name

	 # Constructor. Supports the standard block semantics via yield
	 def initialize(&block)
	    self.type = 'qemu'
	    yield self if block_given?
	 end

	 # Given an Hypervisor object, convert into a list of qemuargs commands
	 def Hypervisor(hyper, amd64=true)

	   # Allocate a new array to store values.
	   value = Array.new

	   # If CPU hot plug is enabled, then current cpu count < max cpu count
	   count = hyper.cpu_count.to_s 
	   if (hyper.cpu_hot_plug)
	      value << ["-smp", "cpus=#{count},maxcpus=255"]
	   else
	      value << ["-smp", "cpus=#{count},maxcpus=#{count}"]
	   end
	  
	   # NOTE: CPU Execution Cap is not implemented in KVM
	   # NOTE: Video RAM Size is not implemented in KVM

	   # RAM Size is always translated into MiB
	   value << ["-m", MiB(hyper.ram_size).to_s]

	   # Memory Balloon size is ignored by KVM. It is just enabling or
	   # disabling a feature.
	   if MiB(hyper.guest_balloon_size) > 0
	      value << ["-balloon", "none"]
	   else
	      value << ["-balloon", "virtio"]
	   end

	   # ACPI is by default enabled on KVM. We just have to handle the 
	   # disabled case.
	   if !hyper.acpi_enabled
	      value << ["-no-acpi"]
	   end

	   # We have to use EFI firmware for virtual machines, if it has been
	   # asked for.
	   if hyper.use_efi
	      value << ["-bios", "/usr/share/ovmf/OVMF.fd"]
	   end

	   # KVM works very differently w.r.t. CPU features. First a base CPU 
	   # (64 bit or 32 bit) has to be selected and then more features need
	   # to be added.
	   cpu_features  =   (amd64)?'qemu64':'qemu32'
	   cpu_features  +=  ',+pae'   if hyper.pae_enabled 
	   cpu_features  +=  ',+vmx'   if hyper.hw_virt_enabled
	   cpu_features  +=  ',+npt'   if hyper.hw_nested_pages
	   cpu_features  +=  ',+pse36' if hyper.use_large_pages
	   cpu_features  +=  ',+hypervisor' if hyper.unrestricted_guest_mode
	   value << ["-cpu", cpu_features]

	   # Set the value for QEMU Args
	   self.qemuargs = value
	 end

      end

   end  # Virtual

end # PackerFiles
