# Requires
require 'Core/Hypervisor'
require 'Utils/HashSerializer'
require 'Utils/Size'

module PackerFiles

   module Virtual

      # Abstraction for VMWare builder in Packer. Refer to Packer.io
      # documentation on more details on the various variables for 
      # VMWare ISO Builder.
      class VMWare < Utils::HashSerializer

	 # Import Size with VirtualBox class as a Mix-in
	 include Utils::Size

	 # List of attributes that would be converted into Hash Values. The Hash
	 # values will eventually be converted into JSON values.
	 hash_variable :iso_checksum
	 hash_variable :iso_checksum_type
	 hash_variable :iso_url
	 hash_variable :ssh_username
	 hash_variable :boot_command, Array
	 hash_variable :boot_wait
	 hash_variable :disk_size,    Integer
	 hash_variable :disk_type_id
	 hash_variable :floppy_files, Array
	 hash_variable :fusion_app_path
	 hash_variable :guest_os_type
	 hash_variable :headless
	 hash_variable :http_directory
	 hash_variable :http_port_min, Integer
	 hash_variable :http_port_max, Integer
	 hash_variable :iso_urls, Array
	 hash_variable :output_directory
	 hash_variable :remote_cache_datastore
	 hash_variable :remote_cache_directory
	 hash_variable :remote_datastore
	 hash_variable :remote_host
	 hash_variable :remote_password
	 hash_variable :remote_type
	 hash_variable :remote_username
	 hash_variable :shutdown_command
	 hash_variable :shutdown_timeout
	 hash_variable :ssh_host
	 hash_variable :ssh_key_path
	 hash_variable :ssh_password
	 hash_variable :ssh_port, Integer
	 hash_variable :ssh_wait_timeout
	 hash_variable :tools_upload_flavor
	 hash_variable :tools_upload_path
	 hash_variable :vm_name
	 hash_variable :vmdk_name
	 hash_variable :vmx_data, Hash
	 hash_variable :vmx_data_post, Hash
	 hash_variable :vmx_template_path
	 hash_variable :vnc_port_min, Integer
	 hash_variable :vnc_port_max, Integer
	 hash_variable :type
	 hash_variable :name

	 # Constructor. Supports the standard block semantics via yield
	 def initialize(&block)
	    self.type = 'vmware-iso'
	    yield self if block_given?
	 end

	 # Given an Hypervisor object, convert into a list of VMX commands
	 # and set vmx_data accordingly
	 def Hypervisor(hyper)
	   value = Hash.new
	   value['numvcpus']      = hyper.cpu_count.to_s
	   value['vcpu.hotadd']   = true_false(hyper.cpu_hot_plug)
	   value['memsize']       = MiB(hyper.ram_size).to_s
	   value['mem.hotadd']    = 'TRUE'
	   value['svga.vramsize'] = Bytes(MiB(hyper.video_ram_size)).to_s
	   value['acpi.present']  = true_false(hyper.acpi_enabled)
	   value['paevm']         = true_false(hyper.pae_enabled)
	   value['vhv.enable']    = true_false(hyper.hw_virt_enabled)
	   value['monitor_control.disable_mmu_largepages'] = true_false(!hyper.use_large_pages)
	   value['mks.enable3d']  = true_false(hyper.video_3d_acceleration)
	   value['firmware']      = 'efi' if (hyper.use_efi)
	   value['sched.mem.maxmemctl'] = MiB(hyper.guest_balloon_size)

	   # CPU Speed limit is done by converting any value into MHZ
	   value['sched.cpu.max']   = MiB(hyper.cpu_speed)
	   value['sched.cpu.units'] = 'mhz'
	   
	   #NOTE: CPU Execution Cap is not supported in VMX File settings. 
	   #NOTE: Extended Page Table [Nested Paging] is enabled because of VHV.
	   #NOTE: Unrestricted  Guest Mode is enabled because of VHV.
	   self.vmx_data = value
	 end

	# Convert a boolean to a TRUE/FALSE string
	private
	def true_false(value)
	   return "TRUE"  if (value)
	   return "FALSE" if (!value)
	end


      end

   end  # Virtual

end # PackerFiles
