# Requires
require 'Core/Hypervisor'
require 'Utils/HashSerializer'
require 'Utils/Size'

module PackerFiles

   module Virtual

       # Abstraction for VirtualBox builder in Packer. Refer to Packer.io
       # documentation on more details on the various variables for 
       # VirtualBox ISO Builder.
       class VirtualBox < Utils::HashSerializer

	  # Import Size with VirtualBox class as a Mix-in
	  include Utils::Size

	  # List of attributes that would be converted into Hash Values. The Hash
	  # values will eventually be converted into JSON values.
	  hash_variable :iso_checksum
	  hash_variable :iso_checksum_type
	  hash_variable :iso_url
	  hash_variable :ssh_username
	  hash_variable :boot_command, Array
	  hash_variable :boot_wait
	  hash_variable :disk_size,    Integer
	  hash_variable :export_opts, Array
	  hash_variable :floppy_files, Array
	  hash_variable :format
	  hash_variable :guest_additions_mode
	  hash_variable :guest_additions_path
	  hash_variable :guest_additions_sha256
	  hash_variable :guest_additions_url
	  hash_variable :guest_os_type
	  hash_variable :hard_drive_interface
	  hash_variable :headless
	  hash_variable :http_directory
	  hash_variable :http_port_min, Integer
	  hash_variable :http_port_max, Integer
	  hash_variable :iso_interface
	  hash_variable :iso_urls, Array
	  hash_variable :output_directory
	  hash_variable :shutdown_command
	  hash_variable :shutdown_timeout
	  hash_variable :ssh_host_port_min, Integer
	  hash_variable :ssh_host_port_max, Integer
	  hash_variable :ssh_key_path
	  hash_variable :ssh_password
	  hash_variable :ssh_port, Integer
	  hash_variable :ssh_wait_timeout
	  hash_variable :vboxmanage, Array
	  hash_variable :vboxmanage_post, Array
	  hash_variable :virtualbox_version_file
	  hash_variable :vm_name
	  hash_variable :type
	  hash_variable :name

	  # Constructor. Supports the standard block semantics via yield
	  def initialize(&block)
	     self.type = 'virtualbox-iso'
	     yield self if block_given?
	  end

	  # Given an Hypervisor object, convert into a list of VBoxmanage commands
	  # and set vboxmanage attribute accordingly.
	  def Hypervisor(hyper)
	    value = Array.new
	    std   = ["modifyvm", "{{.Name}}"]
	    value << std + ["--cpus",               hyper.cpu_count.to_s]
	    value << std + ["--cpuhotplug",         on_off(hyper.cpu_hot_plug)]
	    value << std + ["--cpuexecutioncap",    hyper.cpu_execution_cap.to_s]
	    value << std + ["--memory",             MiB(hyper.ram_size).to_s]
	    value << std + ["--vram",               MiB(hyper.video_ram_size).to_s]
	    value << std + ["--acpi",               on_off(hyper.acpi_enabled)]
	    value << std + ["--pae",                on_off(hyper.pae_enabled)]
	    value << std + ["--hwvirtex",           on_off(hyper.hw_virt_enabled)]
	    value << std + ["--nestedpaging",       on_off(hyper.hw_virt_enabled)]
	    value << std + ["--vtxvpid",            on_off(hyper.hw_virt_enabled)]
	    value << std + ["--largepages",         on_off(hyper.use_large_pages)]
	    value << std + ["--vtxux",              on_off(hyper.unrestricted_guest_mode)]
	    value << std + ["--accelerate3d",       on_off(hyper.video_3d_acceleration)]
	    value << std + ["--firmware","efi"]     if (hyper.use_efi)
	    value << std + ["--guestmemoryballoon", MiB(hyper.guest_balloon_size).to_s]
	    self.vboxmanage = value
	  end

	 # Convert a boolean to a on/off string
	 private
	 def on_off(value)
	    return "on"  if (value)
	    return "off" if (!value)
	 end

       end # VirtualBox

   end  # Virtual

end # PackerFiles
